/*****************************************************************************\

    Filename :			CLREDIT.PRG / CLREPORT.PRG
    Version / Date :		3.00 / 11.1990

    Purpose :			report editor

    Author / (C) :		Huggle / Dr. Huggle & Partner GmbH

\*****************************************************************************/

#include "extkey.ch"
#include "clrepgen.ch"
#include "atomedit.ch"


function ClrEditor (cLib, cRepo)

#ifdef GERMAN
  static cMess   := "Bei der Arbeit", ;
	 cError1 := "DOS-Fehler oder TEMP / TMP falsch gesetzt", ;
	 cError2 := "Fataler Fehler: Report Daten inkonsistent", ;
	 cError3 := "Speichermangel: Report ist nicht zu speichern"
#else
  static cMess   := "Processing", ;
	 cError1 := "DOS error or invalid TMP / TEMP directory", ;
	 cError2 := "Fatal error: inconsistent report data", ;
	 cError3 := "Memory failure: report cannot be saved"
#endif
  local bStateUDF := {|nAct,xP1,xP2,xP3| EditorState (nAct, xP1, xP2, xP3)}, ;
	bQueryUDF := {|nQuery,xParam| EditorQuery (nQuery, xParam)}, ;
	bKeyUDF   := {|nKey| EditorKeys (nKey)}, ;
	cText, cTmpFile, lSave, i, svCursor, svColor

  private V_HSTW := "MAIN EDIT"

  /* FIELD_MAP : V_FIELDPOS[] < N >
  **
  ** field reference indirection array
  */
  private V_FIELDPOS

  /* LINE_INFO : V_LINEDATA[][5] < C,C,C,L,L,L >
  **
  ** line expression, line attribute, suppress empty line, do not divide
  ** record though page change, always print text mask
  */
  private V_LINEDATA

  /* DBF_INFO : V_DBFDATA[][35]	< C,C,C,C,L,C,C,C,C,L,L { C,C,C } >
  **
  ** filename, alias, index file, index expression, always reindex,
  ** index type, filter expression, start expression, while expression,
  ** deleted flag, skip record if relation failed
  **
  ** Eight times relation: expression, child alias, type
  */
  private V_DBFDATA

  /* GROUP_INFO : V_GROUPDATA[][6] < C,L,L,L,L,L >
  **
  ** group expression, eject on group change, reset page number,
  ** coherent group head, coherent group foot, additional head
  */
  private V_GROUPDATA

  /* PAGE_INFO : V_PAGEINFO[15] */
  private V_PAGEINFO

  /* REPO_INFO : V_REPOINFO[8] */
  private V_REPOINFO

  private V_REPONAME := cRepo
  private V_LIBNAME  := cLib


  ClrMessage (cMess, 3)			/*********** load report *************/

  if ( (cTmpFile := LIB->(__repLoad (cRepo, "REPORT", @cText, @V_FIELDPOS, ;
				    @V_DBFDATA, @V_LINEDATA, @V_GROUPDATA, ;
				    @V_PAGEINFO, @V_REPOINFO))) == NIL )
    ClrWindow ()	/* wait window */
    ClrMessage (cError1, 1)
    return NIL
  end

  if ( len (V_LINEDATA) == 0 )			/* new report */
    aadd (V_LINEDATA, { ClrStrRPad (__repXchf ("L", _XCHF_Default), 3), ;
			[], [], .f., .f., .f. })
  else						/* translate */
    i := 0
    AEval (V_LINEDATA, ;
           {|aData| ++i, ;
		V_LINEDATA[i][1] := ;
		   ClrStrRPad (__repXchf ("L", _XCHF_National, aData[1]), 3)})
  end

  ClrWindow ()		/* wait window */

//  svCursor := SetCursor (SC_NORMAL)	/******** start report editor ********/
  svColor := SetColor (CLRCOLOR[_C_EDITOR])

  if ( (lSave := __repEditor (@cText, 3, 4, MaxRow () - 1, MaxCol (), ;
			CLRCOLOR[_C_EDITOR], @V_FIELDPOS, @V_LINEDATA, ;
			bKeyUDF, bQueryUDF, bStateUDF)) == NIL )
    ClrMessage (cError2, 1) ; lSave := .f. ; end	/* data error */

  if ( lSave .and. cText == NIL )			/* memory error */
    ClrMessage (cError3, 1) ; lSave := .f. ; end

  SetColor (svColor) //; SetCursor (svCursor)

  if ( lSave )				/************ save changes ***********/
    ClrMessage (cMess, 3)

    i := 0					/* translate */
    AEval (V_LINEDATA, ;
	   {|aData| ++i, ;
		    V_LINEDATA[i][1] := ;
			Trim (__repXchf ("L", _XCHF_System, aData[1]))})

    REPORT->(__repStore (cTmpFile, cRepo, "LIB", cText, V_FIELDPOS, ;
		V_DBFDATA, V_LINEDATA, V_GROUPDATA, V_PAGEINFO, V_REPOINFO))

    ClrWindow ()	/* wait window */
  else
    REPORT->(__repCancel (cTmpFile)) ; end

return NIL


**********< Show row during line type editing >********************************

function ClrShowRow (nItem)

  local svColor

  svColor := SetColor (CLRCOLOR[_C_EDITOR])

  EditorState (_ESTATE_POSITION, nItem - 1, 0)

  SetColor (svColor)

return NIL


*******************************************************************************
**********< Callback functions >***********************************************
*******************************************************************************

**********< Status function EditorState() >************************************
*
* This function is called by __repEditor() when a status has changed.
*
* Syntax:	logVar := EditorState (nAct, xMisc1, xMisc2)
*
* Parameters:
*    <nAct> specifies the desired action:
*  	_ESTATE_START	 - editor start
*	_ESTATE_POSITION	 - cursor has been moved on a field
*	_ESTATE_ONFIELD	 - cursor has been moved off a field
*	_ESTATE_OFFIELD	 - row / column has changed
*	_ESTATE_RULER	 - display ruler
*	_ESTATE_INSERT	 - the insert mode has changed
*	_ESTATE_ATTRIBUT	 - current attribute has changed
*	_ESTATE_CHATTR	 - current character attribute has changed
*	_ESTATE_WAITSTART - wait state start
*	_ESTATE_WAITEND	 - wait state end
*	_ESTATE_LNTYPE	 - display line type
*	_ESTATE_END	 - editor exit
*	_ESTATE_TONE	 - send signal / alarm
*    <xMisc1>, <xMisc2>, <xMisc3>: additional information (different types)
*
* Return:	NIL
*

static func EditorState (nAct, xMisc1, xMisc2, xMisc3, xMisc4)

#ifdef GERMAN
  static cInsert    := "Einf", ;
	 aAttribute := { "Fett ", "Kurs ", "Hoch ", "Tief " }, ;
	 aChAttr    := { "F", "K", "H", "T" }, ;
	 cMess      := "Bei der Arbeit"
  local  bDisplRC   := {|r,c| QQout ("Z:" + str (xMisc1 + 1, 4) + "  " + ;
				     "K:" + str (xMisc2 + 1, 4)) }
#else
  static cInsert    := "Ins", ;
	 aAttribute := { "Bold ", "Ital ", "Sups ", "Subs " }, ;
	 aChAttr    := { "B", "I", "H", "L" }, ;
	 cMess      := "Processing"
  local  bDisplRC   := {|r,c| QQout ("R:" + str (xMisc1 + 1, 3) + "  " + ;
				     "C:" + str (xMisc2 + 1, 3)) }
#endif
  static cScore := []	/* symbol string for ruler */
  local i, nStart, nEnd, nFail, cStr

  do case
    case nAct == _ESTATE_POSITION	/* row / column has changed */
      * xMisc1 = current row, xMisc2 = current column
      SetPos (1, 0) ; Eval (bDisplRC, xMisc1, xMisc2)

    case nAct == _ESTATE_ONFIELD	/* cursor has been moved on field */
      * xMisc1 = record number of actual field in file REPORT
      if ( REPORT->(Go (xMisc1)) )
        SelColor (3)
        @ 1, 41 say Left (Trim (REPORT->field_name) + "  " + ;
			  REPORT->field_expr, MaxCol () - 40)
        SelColor (1)
      end

    case nAct == _ESTATE_OFFIELD	/* cursor has been moved off field */
      * no additional parameters
      @ 1, 41

    case nAct == _ESTATE_RULER		/* display ruler */
      * xMisc1 = first visible column, xMisc2 = window width,
      * xMisc3 = editor window left column
      nEnd := iif ((nStart := xMisc1) + xMisc2 < len (cScore), ;
		   nStart + xMisc2, len (cScore))

      if ( nEnd < nStart + 3 )
        @ 2, xMisc3 to 2, xMisc3 + xMisc2 - 1 ; @ 2, xMisc3 say chr (17)
      else
        @ 2, xMisc3 say iif (nStart > 0, chr (17), "[") + ;
			SubStr (cScore, nStart + 2, nEnd - nStart - 2) + ;
			iif (len (cScore) > nEnd, chr (16), "]")
        if ( (nEnd -= nStart) < xMisc2 )
          @ 2, nEnd + xMisc3 to 2, xMisc2 + xMisc3 - 1 ; end
      end

    case nAct == _ESTATE_LNTYPE		/* display linetypes */
      * xMisc1 = first visible line (from 0), xMisc2 = window height,
      * xMisc3 = editor window top row
      if ( (nFail := len (V_LINEDATA) - (xMisc1 + xMisc2)) < 0 )
        Scroll (xMisc2 + xMisc3 + nFail, 0, xMisc2 + xMisc3 - 1, 2, 0)
        xMisc2 += nFail
      end

      SelColor (2)
      AEval (V_LINEDATA, {|aData| SetPos (xMisc3++, 0), QQout (aData[1])}, ;
	     xMisc1 + 1, xMisc2)
      SelColor (1)

    case nAct == _ESTATE_INSERT		/* insert mode has changed */
      * xMisc1 = current insert state
      @ 1, 34 say iif (xMisc1, cInsert, "    ")

    case nAct == _ESTATE_ATTRIBUT	/* current attribute has changed */
      * xMisc1 = attribute array
      Scroll (1, 17, 1, 30, 0) ; SetPos (1, 17) ; i := 0
      AEval (xMisc1, {|lOn| ++i, QQout (iif (lOn, aAttribute[i], []))})

    case nAct == _ESTATE_CHATTR		/* character attribute has changed */
      * xMisc1 = attribute array; NIL if on a field
      if ( xMisc1 != NIL )
        cStr := '[' ; i := 0 ; SetPos (1, 41)
        AEval (xMisc1, {|lOn| ++i, cStr += iif (lOn, aChAttr[i], [])})
        ?? iif (Len (cStr) == 1, "     ", Left (cStr + "]  ", 5))
      end

    case nAct == _ESTATE_WAITSTART	/* wait state start */
      * no additional parameters
      ClrMessage (cMess, 3)

    case nAct == _ESTATE_WAITEND	/* waitstate end */
      * no additional parameters
      ClrWindow ()

    case nAct == _ESTATE_START		/* editor start */
      * no additional parameters
      Scroll (1, 0, MaxRow () - 1, MaxCol (), 0)	/* clear window */
      @ 2, 0 to 2, MaxCol () ; @ 3, 3 to MaxRow () - 1, 3 ; @ 2, 3 say ""

      cScore := ;
	 ".........10........20........30........40........50........" + ;
	"60........70........80........90........100.......110......." + ;
        "120.......130.......140.......150.......160.......170......." + ;
        "180.......190.......200.......210.......220.......230......." + ;
        "240.......250..."

    case nAct == _ESTATE_TONE		/* send signal */
      * xMisc1 = tone type
      ClrBeep ()

    case nAct == _ESTATE_END		/* editor exit */
      * no additional parameters
  end

return NIL


static func EditorQuery (nAct, xParam)

#ifdef GERMAN
  static cQuery := "Zeile enthlt Felder: Lschen", ;
	 cSay1  := "Suchbegriff:", ;
	 cSay2  := "Zeilennummer:"
#else
  static cQuery := "Line contains fields, delete", ;
	 cSay1  := "Search token:", ;
	 cSay2  := "Line number:"
#endif
  local xResult, nInput := 0, cInput := Space (40)

  do case
    case nAct == _EQUERY_DELETE		/* erase line containing fields */
      xResult := ClrYNQuery (NIL, NIL, .t., /* color */, cQuery, .t.)
    case nAct == _EQUERY_SEARCH		/* query search string */
      if ( ClrAccept (NIL, NIL, .t., /* color */, NIL, cSay1, @cInput) )
        xResult := Trim (cInput) ; end
    case nAct == _EQUERY_GOTO		/* query line number */
      if ( ClrAccept (NIL, NIL, .t., NIL, NIL, cSay2, @nInput, "999") )
        xResult := nInput ; end
    case nAct == _EQUERY_TEMPLATE	/* field pattern for field <xParam> */
      xResult := ClrTemplate (xParam)
  end

return xResult


**********< Exception-key function EditorKeys() >******************************
*
* This function is called by __repEditor() each time one key has been pressed.
*
* Syntax:	numVar := EditorKeys (nKey)
*
* Return:	Numeric value; old/new key code, 0 to discard key
*

static func EditorKeys (nKey)

  if ( nKey == K_F1 )
    __repExec (_EXEC_SETFOCUS, .f.)
    CallHelp () ; nKey := 0
    __repExec (_EXEC_SETFOCUS, .t.)
  elseif ( nKey == K_ESC )
    __repExec (_EXEC_SETFOCUS, .f.)
    nKey := WorkMenu ()
    __repExec (_EXEC_SETFOCUS, .t.)
  elseif ( nKey == K_F10 )
    __repExec (_EXEC_SETFOCUS, .f.)
    ClrChooseFld () ; nKey := 0
    __repExec (_EXEC_SETFOCUS, .t.)
  elseif ( nKey == K_F5 )
    nKey := BoxSign( .F. )
  elseif ( nKey == K_SH_F5 )
    __repExec (_EXEC_SETFOCUS, .f.)
    BoxSign( .T. )
    __repExec (_EXEC_SETFOCUS, .t.)
  end

return nKey


Static Function BoxSign( lSet )

#ifdef GERMAN
  static cMess := "Boxzeichen"
#else
  static cMess := "box character"
#endif

	Static nRow  := 1
	Static nCol  := 2
	Static aSign := { "Ŀ ͻ", ;
			  "     ", ;
			  "Ĵ ͹", ;
			  " ͼ", ;
			  "͸ ķ", ;
			  "     ", ;
			  "͵ Ķ", ;
			  "; Ľ"  }
	Local  nSign   := 0
	Local  nRowOld := nRow
	Local  nColOld := nCol
	Local  nKey

	If ( lSet )
	   ClrWindow( 7, 32, 18, 48, CLRCOLOR[ _C_LIST ], cMess )
	   AEval( aSign, { | line, i | DevPos( 8 + i, 35 ), DevOut( line ) } )

	   
	   While ( .T. )
	      @ 8 + nRow, 34 + nCol Say SubStr( aSign[ nRow ], nCol, 1 ) ;
	        Color StrToken( CLRCOLOR[ _C_LIST ], ",", 2 )

	      nKey := Inkey( 0 )
	      
	      Do Case
	         Case ( nKey == K_ESC .Or. nKey == K_RETURN )
		      Exit
		 Case ( nKey == K_UP .And. nRow > 1 )
		      nRow --
		 Case ( nKey == K_DOWN .And. nRow < Len( aSign ) )
		      nRow ++
		 Case ( nKey == K_LEFT .And. nCol > 1 )
		      nCol --
		 Case ( nKey == K_RIGHT .And. nCol < Len( aSign[ 1 ] ) )
		      nCol ++
		 Case ( nKey == K_HOME )
		      nCol := 1
		 Case ( nKey == K_END )
		      nCol := Len( aSign[ 1 ] )
		 Case ( nKey == K_PGUP )
		      nRow := 1
		 Case ( nKey == K_PGDN )
		      nRow := Len( aSign )
	      EndCase

	      @ 8 + nRowOld, 34 + nColOld ;
	        Say   SubStr( aSign[ nRowOld ], nColOld, 1 ) ;
	        Color CLRCOLOR[ _C_LIST ]

	      nRowOld := nRow
	      nColOld := nCol
	   End
	   
	   ClrWindow()

	Else
	   nSign := Asc( SubStr( aSign[ nRow ], nCol, 1 ) )
	EndIf
	
	Return ( nSign )


*******************************************************************************
**********< Supplement functions >*********************************************
*******************************************************************************

static func WorkMenu ()

#ifdef GERMAN
  static aMenu := { "Felder            ", ;
		    "Gruppen           ", ;
		    "Datenbanken       ", ;
		    "Zeilendaten       ", ;
		    "", ;
		    "Bereiche          ", ;
		    "", ;
		    "Whle Feld        ", ;
		    "", ;
		    "Editor            ", ;
		    "bertragen        ", ;
		    "Lsche gesamt     ", ;
		    "", ;
		    "Seitendaten       ", ;
		    "Pfade / Drucker   ", ;
		    "", ;
		    "Ausgabe Definition", ;
		    "", ;
		    "Hilfe             " }
#else
  static aMenu := { "Fields            ", ;
		    "Groups            ", ;
		    "Databases         ", ;
		    "Line Data         ", ;
		    "", ;
		    "Areas / Line Types", ;
		    "", ;
		    "Place Field       ", ;
		    "", ;
		    "Editor            ", ;
		    "Transfer          ", ;
		    "Clear mask data   ", ;
		    "", ;
		    "Page Format       ", ;
		    "Supplement/Printer", ;
		    "", ;
		    "Output Definition ", ;
		    "", ;
		    "Help              " }
#endif
  static aAva := { .t., .t., .t., .t., .f., .t., .f., .t., .f., ;
		   .t., .t., .t., .f., .t., .t., .f., .t., .f., .t. }
  local aMenuExec := { {|| ClrField () }, {||  ClrGroup () }, ;
		       {|| ClrDbf () }, {|| ClrLine () }, ;
		       {|| NIL }, {|| ClrLineType () }, ;
		       {|| NIL }, {|| ClrChooseFld () }, ;
		       {|| NIL }, {||  EditMenu () }, ;
		       {|| Transfer () }, {|| EraseAll () }, ;
		       {|| NIL }, {|| ClrGlobPage (V_PAGEINFO) }, ;
		       {|| ClrGlobGlob (V_REPOINFO) }, {|| NIL }, ;
		       {|| ClrOutDef () }, {|| NIL }, ;
		       {|| CallHelp () } }, ;
	nChoice, nResult, lLeave := .f.

  private V_HSTW := "REPOEDIT MENU"

  __repExec (_EXEC_SETVIEW, NIL, NIL, NIL, 53)

  Scroll (2, 54, MaxRow () - 1, MaxCol (), 0)

  if ( (nChoice := ClrAchoice ({ 2, 56, 22, 77 }, .f., CLRCOLOR[_C_EXTRA], ;
			       /* head */, /* foot */, /* flags */, aMenu, ;
			       aAva)) != 0 )
    nResult := Eval (aMenuExec[nChoice]) ; end

  ClrWindow ()		/* menu window */

  __repExec (_EXEC_SETVIEW, NIL, NIL, NIL, MaxCol ())

return iif (nResult == NIL, 0, nResult)


**********< supplement functions >*********************************************

static func EditMenu (oEditor)		/* editor commands */

#ifdef GERMAN
  static aMenu := { "Gehe zu Zeile", ;
		    "Suchen", ;
		    "<Zeile>", ;
		    "Einfgen", ;
		    "Lschen", ;
		    "<Block>", ;
		    "Markieren", ;
		    "Kopieren", ;
		    "<Attribute>", ;
		    "Fett an/aus", ;
		    "Italics an/aus", ;
		    "Hoch an/aus", ;
		    "Tief an/aus" }
#else
  static aMenu := { "Go to line", ;
		    "Search", ;
		    "<line>", ;
		    "Insert", ;
		    "Delete", ;
		    "<block>", ;
		    "Mark", ;
		    "Copy", ;
		    "<attribute>", ;
		    "Bold on/off", ;
		    "Italics on/off", ;
		    "High on/off", ;
		    "Low on/off" }
#endif
  static aMapToKey := { K_ALT_G, K_ALT_S, 0, K_CTRL_N, K_ALT_U, 0, ;
			K_ALT_M, K_CTRL_J, 0, K_ALT_B, K_ALT_Y, ;
			K_ALT_T, K_ALT_V }
  static aAva := { .t., .t., .f., .t., .t., .f., .t., .t., .f., ;
		   .t., .t., .t., .t. }
  local nChoice, nResult

  private V_HSTW := "EDITOR MENU"

  if ( (nChoice := ClrAchoice ({ 4, 31, NIL, NIL }, .t., /* color */, ;
			       /* head */, /* foot */, /* flags */, ;
			       aMenu, aAva)) != 0 )
    nResult := aMapToKey[nChoice] ; end

return nResult


static func Transfer (oEditor)		/* save / lose changes */

#ifdef GERMAN
  static aPrompt := { "Speichern", "Verwerfen" }
  local  cQuery  := "nderungen in Report " + V_REPONAME + " speichern"
#else
  static aPrompt := { "Save", "Lose" }
  local  cQuery  := "Save changes in report " + V_REPONAME
#endif
  local nChoice

  private V_HSTW := "REPORT SAVE"

  if ( (nChoice := ClrQuery (NIL, NIL, .t., NIL, cQuery, aPrompt)) != 0 )
    __repExec (_EXEC_DESTROY, nChoice == 1) ; end

return NIL


static func EraseAll (oEditor)		/* global erase mask data */

#ifdef GERMAN
  static cQuery  := "Alle Maskendaten zum Report lschen", ;
         aPrompt := { "Abbrechen", "Lschen" }
#else
  static cQuery  := "Delete all mask-data of report", ;
         aPrompt := { "Abort", "Continue" }
#endif

  private V_HSTW := "GLOB RDEL"

  if ( ClrQuery (NIL, NIL, .t., /* color */, cQuery, aPrompt) == 2 )
    __repExec (_EXEC_GLOBERASE, ;
               { ClrStrRPad (__repXchf ("L", _XCHF_Default), 3), ;
                 [], [], .f., .f., .f. })
  end

return NIL


static func CallHelp ()			/* display On-Line-Help */

  local bKeyBlock := SetKey (K_F1)

  private V_HSTW := "MAIN EDIT"

  if ( bKeyBlock != NIL )
    Eval (bKeyBlock) ; end

return NIL


/*----------------------------< EOF clredit.prg >----------------------------*/
